
#include <i86.h>
#include <dos.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>

#include "vdt.h"
#include "global.h"
#include "cpart3.h"

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

int cpart3::Startup() {

	world.light.ambient = 30;
	world.light.max     = 60;
	world.light.FollowCamera = YES;
	world.camera.Move(0,0,0);
	world.camera.SetAngles(0,0,0);
	world.camera.zoom = 1;
     #ifdef datafile
	dl.Seek("kopule.pal");
	world.pal.Load(dl.LibFile);
	dl.Seek("kopule.shd");
	world.shade.Load(dl.LibFile);
     #else
	world.pal.Load("..\\data\\kopule.pal");
	world.shade.Load("..\\data\\kopule.shd"); //it is necessary!!!!!
     #endif

     #ifdef datafile
	dl.Seek("kopule.vsc");
	world.MergeMesh("scene2",dl.LibFile);
     #else
	world.MergeMesh("scene2","..\\data\\kopule.vsc");
     #endif
	world.SetTwoSided(OFF);

        //-- [ PATH INIT ] ------------------------------------------------
     #ifdef datafile
	dl.Seek("kopule.pth");
	fread((void*)step,1,dl.ActualSize(),dl.LibFile);
	steps = dl.ActualSize()/sizeof(steps_struct);
     #else
	FILE* f = fopen("..\\data\\kopule.pth","rb");
	int delka=filesize(f);
	fread((void*)step,1,delka,f);
	fclose(f);
	steps = delka/sizeof(steps_struct);
     #endif
        compute_movie();

	// --- [ TEXTURES ] ------------------------------------------------
     #ifdef datafile
	dl.Seek("kopule.pcx");
	textura1.Load(dl.LibFile);
	dl.Seek("vozidlo.pcx");
	textura2.Load(dl.LibFile);
     #else
	textura1.Load("..\\data\\kopule.pcx");
	textura2.Load("..\\data\\vozidlo.pcx");
     #endif
	//remaping textura1 palette into world.pal
        textura1.Remap(&world.pal.pal);
        textura1.Resample(256,256);
        textura2.Remap(&world.pal.pal);
        textura2.Resample(256,256);

	pestmn.Create(100,100);
     #ifdef datafile
	dl.Seek("pestmn.pcx");
	pestmn.Load(dl.LibFile);
     #else
	pestmn.Load("..\\data\\pestmn.pcx");
     #endif
	pestmn.Remap(&world.pal.pal);

	return Success;
}


//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

int cpart3::Init() {

  obj = (Mesh*)world.SelectObject("scene2");

  obj->SetFlags(ShadingNone | MappingTextured);
  obj->SetTexture(&textura1);
  world.Scale(1,1,1);
  obj->SetTwoSided(OFF);

  star=(Chunk*)obj->SelectChunk("alus02");
  star->SetTwoSided(OFF);
  star->SetFlags(ShadingNone | MappingEnvironment);
  star->SetTexture(&textura2);

  star2=(Chunk*)obj->SelectChunk("alus04");
  star2->SetTwoSided(OFF);
  star2->SetFlags(ShadingNone | MappingEnvironment);
  star2->SetTexture(&textura2);

  virtscr.Create(320,200);

  return Success;

}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

long cpart3::filesize(FILE *stream) {
   long curpos, length;

   curpos = ftell(stream);
   fseek(stream, 0L, SEEK_END);
   length = ftell(stream);
   fseek(stream, curpos, SEEK_SET);
   return length;
}

//-----------------------------------------------------------------------------
//--------------------------------------------------------------------------------

void cpart3::compute_movie() {

	frames=0;
	float pas = (float)1/(float)maxpp3;
        for (int k=1; k<steps-1; k++) {
                float t = 0;
		for (int i=0; i<maxpp3; i++) {
                        path[frames].x = bspline( step[k-1].x, step[k].x, step[k+1].x, t );
                        path[frames].y = bspline( step[k-1].y, step[k].y, step[k+1].y, t );
                        path[frames].z = bspline( step[k-1].z, step[k].z, step[k+1].z, t );
                        path[frames].a = bspline( step[k-1].a, step[k].a, step[k+1].a, t );
                        path[frames].b = bspline( step[k-1].b, step[k].b, step[k+1].b, t );
                        path[frames].c = bspline( step[k-1].c, step[k].c, step[k+1].c, t );
                        frames++;
                        t += pas;
                }
        }
 celkanim=frames;
 //printf("%d %d %d",steps,celkanim,maxpp3);
 //getch();
}

//--------------------------------------------------------------------------------
//-----------------------------------------------------------------------------

void cpart3::see_picture (float x, float y, float z, float a, float b, float c) {

	world.camera.Move(x,y,z);
	world.camera.Rotate(a,b,c);
	virtscr.Clear();
	world.FlashCamera(&virtscr);
}

//--------------------------------------------------------------------------------
//-----------------------------------------------------------------------------

int cpart3::Terminate() {

	world.Destroy();
	textura1.Destroy();
	textura2.Destroy();
	pestmn.Destroy();

	return Success;
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

int cpart3::Done(){

 virtscr.Destroy();
 return Success;

}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

int cpart3::Run() {

  if (exitflag==1) return Error;
   check();

  svga.Init(mode_320x200);
  svga.Clear();
  virtscr.Clear();
  svga.SetPalette(&world.pal.pal);

  // camera initial params ***********************
  float cax, cay, caz, cx, cy, cz;
  obj->Rotate(0,0,0); // bugfix for some bug in envmapping in 3d-engine :(

  frameCount=frames=0;
  int pos;
  do{
   // camera stuff
   cx = path[frameCount].x;
   cy = path[frameCount].y;
   cz = path[frameCount].z;
   cax = path[frameCount].a;
   cay = path[frameCount].b;
   caz = path[frameCount].c;
   if (frameCount>(celkanim-1)) {
    cx = path[celkanim-(frameCount-celkanim)].x;
    cy = path[celkanim-(frameCount-celkanim)].y;
    cz = path[celkanim-(frameCount-celkanim)].z;
    cax = path[celkanim-(frameCount-celkanim)].a;
    cay = path[celkanim-(frameCount-celkanim)].b;
    caz = path[celkanim-(frameCount-celkanim)].c;
   }
   star->Rotate(0,-angley,0);
   star2->Rotate(0,angley2,0);
   angley=frameCount*pi/110;
   angley2=frameCount*pi/190;

   // take a picture
   see_picture (cx,cy,cz,cax,cay,caz);

   if (frameCount>1000 && frameCount<(celkanim-3000))
    virtscr.TCopy(&pestmn,220,100);

   // display everything
   svga.Copy(&virtscr,0,0);

   if (exitflag==1) return Error;
   check();
    de.music.GetInfo();
    pos=de.music.info->position;

  } while (pos<0x1D);

 return Success;

}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------


