
#include <i86.h>
#include <dos.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>

#include "vdt.h"
#include "global.h"
#include "movpart.h"

Palette             mypal;

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

void* movpart::OpenFLIFile(char const *filename)
{
        FILE            *file;
        FLIHDR          hdr;
        void            *buffer;

        file = fopen(filename, "rb");
        if(file == NULL) {
                return NULL;
        }

        //read header of FLI 
        fread(&hdr, sizeof(FLIHDR), 1, file);
        if(hdr.magic != 0xAF11) {
                return NULL;
        }

	/*
	printf("Frames  : %d\n", hdr.frames);
	printf("Width   : %d\n", hdr.width);
	printf("Height  : %d\n", hdr.height);
	printf("Depth   : %d\n", hdr.depth);
	printf("Flags   : %Xh\n", hdr.flags);
	printf("Size    : %d\n", hdr.size - 128);
	printf("\nReading in entire .FLI file...\n");
	*/

        buffer = (unsigned char*) malloc(hdr.size - 128);
        if(buffer == NULL) {
                printf("Not enough memory to load file.\n");
                fclose(file);
                return NULL;
        }

        fread(buffer, sizeof(unsigned char), hdr.size - 128, file);
	//printf("Buffer read complete\n");

        numframes = hdr.frames;
        waitcount = hdr.speed;

        return buffer;
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

void* movpart::OpenFLIFilelib(void)
{
	FLIHDR          hdr;
        void            *buffer;


	//read header of FLI
	dl.Seek("skate.mv");
	fread(&hdr, sizeof(FLIHDR),1,dl.LibFile);
	if(hdr.magic != 0xAF11) {
		return NULL;
	}

	/*
	printf("Frames  : %d\n", hdr.frames);
	printf("Width   : %d\n", hdr.width);
	printf("Height  : %d\n", hdr.height);
	printf("Depth   : %d\n", hdr.depth);
	printf("Flags   : %Xh\n", hdr.flags);
	printf("Size    : %d\n", hdr.size - 128);
	printf("\nReading in entire .FLI file...\n");
	*/

        buffer = (unsigned char*) malloc(hdr.size - 128);
        if(buffer == NULL) {
                printf("Not enough memory to load file.\n");
		return NULL;
        }

	fread(buffer, sizeof(unsigned char), hdr.size - 128, dl.LibFile);
      //printf("Buffer read complete\n");

        numframes = hdr.frames;
        waitcount = hdr.speed;

        return buffer;
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

BOOL movpart::ReadFLIFrame(FLIFRAME *frame)
{
	memcpy(frame, curptr, sizeof(FLIFRAME));
	curptr+=sizeof(FLIFRAME);
	if(frame->magic != 0xF1FA) {
                printf("Not a frame\n");
                printf("Found %X\n", frame->magic);
                getch();
		return FALSE;
	}

	return TRUE;
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

BOOL movpart::ProcessFrame(FLIFRAME *frame)
{
        DWORD           chunk;
        FLICHUNK        flichunk;

        for(chunk=0; chunk<frame->chunks; chunk++) {
                memcpy(&flichunk, curptr, sizeof(FLICHUNK));
                curptr += sizeof(FLICHUNK);
/*
                printf("Type %X\n", flichunk.type);
*/
                switch(flichunk.type) {
                        case FLI_COLOR:
                                ProcessColorChunk(curptr);
                                break;
                        case FLI_LC:
                                ProcessLcChunk(curptr);
                                break;
                        case FLI_BLACK:
                                ProcessBlackChunk(curptr);
                                break;
                        case FLI_BRUN:
                                ProcessBrunChunk(curptr);
                                break;
                        case FLI_COPY:
                                ProcessCopyChunk(curptr);
                                break;
                        case FLI_DELTA:
                                ProcessDeltaChunk(curptr);
                                break;
                        case FLI_256_COLOR:
                                Process256ColorChunk(curptr);
                                break;
                }

/*
                printf("Size: %d\n", flichunk.size);
                if(getch() == 27)
			return FLS;
*/
                curptr += flichunk.size - sizeof(FLICHUNK);
        }

	return TRUE;
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

void WaitRetrace(LONG count)
{
        while(count--) {
                while(inp(0x3DA) & 8);
                while((inp(0x3DA) & 8) == 0);
        }
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

void movpart::PlayFLIAnimation(void)
{
        DWORD           frame;
        FLIFRAME        fliframe;
        do {
                curptr = flidata;
                for(frame=0; frame<numframes; frame++) {
                        if(!ReadFLIFrame(&fliframe))
                                return;
                        if(!ProcessFrame(&fliframe))
                                return;
                        //memcpy(virtscr.Data,dbuffer,64000);
                        svga.SetPalette(&mypal.pal);
                        svga.Copy(&virtscr,0,0);
                        WaitRetrace(waitcount);
                        if(kbhit()) return;

                }
        } while(loop);
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

int movpart::Startup() {

#ifdef datafile
 flidata=(UBYTE *)OpenFLIFilelib();
#else
 flidata=(UBYTE *)OpenFLIFile("..\\data\\skate.fli");
#endif

 if (!flidata) {
		printf("Error load movie file\n");
		return Error;
 }

 replay.Create(100,100);
#ifdef datafile
 dl.Seek("repl.pcx");
 replay.Load(dl.LibFile);
#else
 replay.Load("..\\data\\repl.pcx");
#endif

 pestsk.Create(100,100);
#ifdef datafile
 dl.Seek("pestsk.pcx");
 pestsk.Load(dl.LibFile);
#else
 pestsk.Load("..\\data\\pestsk.pcx");
#endif

 return Success;
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

int movpart::Init() {

   loop=0;
   return Success;

}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

int movpart::Terminate() {

    free(flidata);
    replay.Destroy();
    pestsk.Destroy();

    return Success;
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

int movpart::Done(){

   return Success;

}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

int movpart::Run() {

 if (exitflag==1) return Error;
   check();

 svga.Init(mode_320x200);
 svga.Clear();
 virtscr.Create(320,200);
 //svga.SetPalette(&mypal.pal);

 /*
 flidata=(UBYTE *)OpenFLIFile("pokus.fli");
 if (!flidata) {
		printf("Error load movie file\n");
		return;
 }
 */

 //PlayFLIAnimation();

 DWORD           frame,frameold,mvcrt;
 FLIFRAME        fliframe;

 frameold=frameCount=0;
 mvcrt=0;
 curptr=flidata;
 int i;
 int inc=2;
 do {
     frame=(int)frameCount;

     if (frame>(frameold+inc)) {
      mvcrt++;
      if (mvcrt>numframes && mvcrt<(numframes+10)) {
	curptr=flidata;
	inc=10;
      }
      frameold=frame;
      if(!ReadFLIFrame(&fliframe))
	 return Error;
      if(!ProcessFrame(&fliframe))
	 return Error;
      svga.SetPalette(&mypal.pal);
      if (mvcrt<numframes) {
	 pestsk.Remap(&mypal.pal);
	 virtscr.TCopy(&pestsk,0,100);
      }
      if (mvcrt>numframes) {
       replay.Remap(&mypal.pal);
       virtscr.TCopy(&replay,210,0);
      }
      svga.Copy(&virtscr,0,0);

      if (exitflag==1) return Error;
      check();
     }

 } while (mvcrt<(2*numframes+8));

 //free(flidata);

 return Success;

}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------


